"""
" Name: acquisition.py
" Author: Cedric Yue Sik Kin, Andrew Ponec
" Description: Script that displays raster patterns on lightcrafter and records data from Labjack
"""

import u3
import matlab.engine
import math 
import scipy.io as sio
import numpy as np
import time
import sys
from dlplc_tcp import *

def connect():
    """Connect to a dlplc or exit"""
    dlplc = LightCrafterTCP()
    if not dlplc.connect():
        print "Unable to connect to device."
        sys.exit(1)
    return dlplc

def display_bitmap(filename, dlplc, i):
    #dlplc = connect()
    #v = dlplc.cmd_current_display_mode(0x00)
    #print "Current display mode: %s"%":".join(c.encode('hex') for c in v.data)

    with open(filename, mode="rb") as f:
        data = f.read()
    #print ":".join(c.encode('hex') for c in data)
    print "Loading image {0} ...".format(i)
    response = dlplc.cmd_static_image(data);
    print "    Done."

    try:
        response.raise_if_error()
    except:
        print "Error:"
        response.show()
        return 1

    #dlplc.close()
    return 0

time.sleep(10)
# connect to running matlab session
# eng = matlab.engine.connect_matlab()

dlplc = connect()

# connect to labjack
labjack = u3.U3()
labjack.configIO(EIOAnalog=3)

# resolution parameters
num_rows = 608
num_cols = 684
res = 32

num_measurements = 256	# for noise filtering
num_patterns = int(res*res)

# initializes matrices that will contain measurements
amplitudes_meas = np.zeros(shape = (num_patterns, num_measurements))
phases_meas = np.zeros(shape = (num_patterns, num_measurements))

#phases_end = []
#amplitudes_end = []

# display patterns and capture labjack values
for i in range(num_patterns):

	pathname = './hadamardPatterns/res{0}/pattern{1}.bmp'.format(res, i)
	#ret = eng.displayPattern(pathname)
	display_bitmap(pathname, dlplc, i)
	time.sleep(0.050) # WAIT 50-100 ms
	#current_phases = []
	#current_amplitudes = []
	#times = []
	#t_end = time.time() + 2
	for k in range(num_measurements):

		amplitudes_meas[i, k] = labjack.getAIN(8)
		phases_meas[i, k] = labjack.getAIN(9)

		#current_phases.append(labjack.getAIN(9))
		#current_amplitudes.append(labjack.getAIN(8))

#phases_end.append(np.mean(current_phases))
#amplitudes_end.append(np.mean(current_amplitudes))

#measurements = {'phases_meas': current_phases, 'amplitudes_meas': current_amplitudes}
#sio.savemat('test3_meas.mat', measurements)

dlplc.close()

result = {'phases_meas' : phases_meas, 'amplitudes_meas' : amplitudes_meas}
sio.savemat('hadamard_capture_res{0}.mat'.format(res), result)