%% Recovery - Raster
% Cedric Yue Sik Kin, Andrew Ponec
%
% Computes the albedo and depth map after scene capture through Hadamard
% reconstruction

clear all; close all;
raw = load('hadamard_capture_res32.mat');

res = 32;

h = hadamard(res*res);
hb = (h + 1) / 2;
hbi = inv(hb);

vmag = mean(raw.amplitudes_meas, 2);
vphs = mean(raw.phases_meas, 2);

db = (vmag - 0.881) / 0.030;
mag = 10.^(db / 20);
phase = (vphs - 0.996) / 0.0107 * pi / 180;

cc = mag .* (cos(phase) + 1i * sin(phase));

ra = hbi * abs(cc);
r = hbi * cc;

ra(1) = mean(ra);
r(1) = mean(r);
magnitude = reshape(abs(r), res, res)';
phase = reshape(angle(r), res, res)';

%% Deconvolution of Phase with TV Prior
addpath('toolbox_optim/toolbox_optim');
addpath('toolbox_optim/toolbox_optim/toolbox/');

y = phase;

lambda = .1;

K = @(x)grad(x);
KS = @(x)-div(x);

Amplitude = @(u)sqrt(sum(u.^2,3));
F = @(u)lambda*sum(sum(Amplitude(u)));
G = @(x)1/2*norm(y-x,'fro')^2;

Normalize = @(u)u./repmat( max(Amplitude(u),1e-10), [1 1 2] );
ProxF = @(u,tau)repmat( perform_soft_thresholding(Amplitude(u),lambda*tau), [1 1 2]).*Normalize(u);
ProxFS = compute_dual_prox(ProxF);

ProxG = @(x,tau)(x+tau*y)/(1+tau);

options.niter = 300;
[xAdmm,EAdmm] = perform_admm(y, K,  KS, ProxFS, ProxG, options);

phase_d = xAdmm;
%%
% Average depth of the bright object.
% m1 = (magnitude > 0.01);
% % mm = arg(r) .* m1;
% % sum(mm) / sum(m1)
% masked_phase = m1 .* phase_d;
% masked_phase(m1 == 0) = min(masked_phase(:)) - 0.1;


%%
% clf;
% figure;
% subplot(221);
% imshow(reshape(ra, res, res)', []);
% 
% subplot(222);
% imshow(masked_phase, []);
% 
% subplot(223);
% imshow(magnitude, []);
% 
% subplot(224);
% imshow(phase, []);

% figure; 
% imagesc(reshape(masked_phase, 16, 16)');
%%
phase_c = phase(:, 4:end);
phase_d_c = phase_d(:, 4:end);
magnitude_c = magnitude(:, 4:end);

tg_min = 0;
tg_max = 1;

curr_min = min(phase_d_c(:));
curr_max = max(phase_d_c(:));
phase_d_r = ((phase_d_c-curr_min)*(tg_max-tg_min))/(curr_max-curr_min) + tg_min;

curr_min = min(phase_c(:));
curr_max = max(phase_c(:));
phase_r = ((phase_c-curr_min)*(tg_max-tg_min))/(curr_max-curr_min) + tg_min;

%% Plotting Intensity Map

mag_plot = magnitude_c.*2;
[oldHeight] = size(mag_plot,1);
newWidth = int32(oldHeight * 16/9);

f = figure;
mag_resized = imresize(mag_plot, [oldHeight newWidth]);
imshow(mag_resized, [], 'InitialMagnification','fit');
saveas(f, 'hadamard_int.png');

%% Assigning Depths to Raw Data

d_square = 1.32;
d_S = 1.57;
d_back = 2.1;

square_ind = [15 19];
back_ind = [30 13];

slope = (d_square - d_back)/(phase_c(square_ind(1),square_ind(2)) ...
    - phase_c(back_ind(1), back_ind(2)));

intercept = d_square - slope * phase_c(square_ind(1),square_ind(2));

depth_map = slope .* phase_c + intercept;
% figure; imagesc(depth_map); colorbar

depth_heatmap = zeros(res, res-3, 3);
depth_heatmap(:,:,1) = (1 - phase_r) ./ 2;
depth_heatmap(:,:,2) = 1;
depth_heatmap(:,:,3) = 1;

depth_heatmap = hsv2rgb(depth_heatmap);

[oldHeight] = size(depth_heatmap, 1);
newWidth = int32(oldHeight * 16/9);
depth_heatmap = imresize(depth_heatmap, [oldHeight newWidth]);

% Heat Map of Depths - Raw
f = figure; colormap('hsv'); 
imshow(depth_heatmap, [], 'InitialMagnification','fit'); 
caxis([min(depth_map(:)) max(depth_map(:))]); 
cb = colorbar; ylabel(cb, '( in m )'); set(gca,'fontsize',18, 'FontWeight','Bold')
saveas(f, 'hadamard_depth_raw.png');

depth_heatmap_c = zeros(res, res-3, 3);
depth_heatmap_c(:,:,1) = (1 - phase_r) ./ 2;
depth_heatmap_c(:,:,2) = 1;
depth_heatmap_c(:,:,3) = magnitude_c.*30;

depth_heatmap_c = hsv2rgb(depth_heatmap_c);

[oldHeight] = size(depth_heatmap_c, 1);
newWidth = int32(oldHeight * 16/9);
depth_heatmap_c = imresize(depth_heatmap_c, [oldHeight newWidth]);

% Heat Map of Depth Combined with Amplitude
f = figure; colormap('hsv'); 
imshow(depth_heatmap_c, [], 'InitialMagnification','fit'); 
caxis([min(depth_map(:)) max(depth_map(:))]); 
cb = colorbar; ylabel(cb, '( in m )'); set(gca,'fontsize',18, 'FontWeight','Bold')
saveas(f, 'hadamard_comb_raw.png');



%% Assigning Depths to Denoised Data

slope = (d_square - d_back)/(phase_d_c(square_ind(1),square_ind(2)) ...
    - phase_d_c(back_ind(1), back_ind(2)));

intercept = d_square - slope * phase_d_c(square_ind(1),square_ind(2));

depth_map_d = slope .* phase_d_c + intercept;
% figure; imagesc(depth_map); colorbar

depth_heatmap_d = zeros(res, res-3, 3);
depth_heatmap_d(:,:,1) = (1 - phase_d_r) ./ 2;
depth_heatmap_d(:,:,2) = 1;
depth_heatmap_d(:,:,3) = 1;

depth_heatmap_d = hsv2rgb(depth_heatmap_d);

[oldHeight] = size(depth_heatmap_d, 1);
newWidth = int32(oldHeight * 16/9);
depth_heatmap_d = imresize(depth_heatmap_d, [oldHeight newWidth]);

% Heat Map of Depths - Denoised
f = figure; colormap('hsv'); 
imshow(depth_heatmap_d, [], 'InitialMagnification','fit'); 
caxis([min(depth_map_d(:)) max(depth_map_d(:))]); 
cb = colorbar; ylabel(cb, '( in m )'); set(gca,'fontsize',18, 'FontWeight','Bold')
saveas(f, 'hadamard_depth_den.png');

depth_heatmap_d_c = zeros(res, res-3, 3);
depth_heatmap_d_c(:,:,1) = (1 - phase_d_r) ./ 2;
depth_heatmap_d_c(:,:,2) = 1;
depth_heatmap_d_c(:,:,3) = magnitude_c.*30;

depth_heatmap_d_c = hsv2rgb(depth_heatmap_d_c);

[oldHeight] = size(depth_heatmap_d_c, 1);
newWidth = int32(oldHeight * 16/9);
depth_heatmap_d_c = imresize(depth_heatmap_d_c, [oldHeight newWidth]);

% Heat Map of Depths Combined With Intensity - Denoised
f = figure; colormap('hsv'); 
imshow(depth_heatmap_d_c, [], 'InitialMagnification','fit'); 
caxis([min(depth_map_d(:)) max(depth_map_d(:))]); 
cb = colorbar; ylabel(cb, '( in m )'); set(gca,'fontsize',18, 'FontWeight','Bold')
saveas(f, 'hadamard_comb_den.png');
